#include "PyEMClasses.h"
#include "Cylinder.h"
#include <boost/python.hpp>
#include <boost/python/to_python_converter.hpp>

namespace python = boost::python;

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(print_overloads, print, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_toInventor, toInventor, 0, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_toDejaVu, toDejaVu, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_toCosec, toCosec, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_if_merge, if_merge, 1, 3);

void export_Cylinder()
{
    python::class_<Cylinder> Cylinder_class("Cylinder", "Cylinder()\n construct an empty Cylinder instance\n\nCylinder(Vect center, Vect axisdir, len = 5.4, r = 2.5, w = 0.9)");
    Cylinder_class.def(python::init<>());
    Cylinder_class.def(python::init<const Vect&, const Vect&, float, float, float>());

    Cylinder_class.def("Center", &Cylinder::Center, 
		       "Vect Center()");
    Cylinder_class.def("Axis", &Cylinder::Axis, 
		       "Vect Axis()");
    Cylinder_class.def("QuaterForAxis", &Cylinder::QuaterForAxis, 
		       "Quater QuaterForAxis()");
    Cylinder_class.def("Xc", &Cylinder::Xc, 
		       "float Xc()");
    Cylinder_class.def("Yc", &Cylinder::Yc, 
		       "float Yc()");
    Cylinder_class.def("Zc", &Cylinder::Zc, 
		       "float Zc()");
    Cylinder_class.def("Alt", &Cylinder::Alt, 
		       "float Alt()");
    Cylinder_class.def("Az", &Cylinder::Az, 
		       "float Az()");
    Cylinder_class.def("Radius", &Cylinder::Radius, 
		       "float Radius()");
    Cylinder_class.def("Length", &Cylinder::Length, 
		       "float Length()");
    Cylinder_class.def("Weight", &Cylinder::Weight, 
		       "float Weight()");
    Cylinder_class.def("StartID", &Cylinder::StartID, 
		       "int StartID()");
    Cylinder_class.def("EndID", &Cylinder::EndID, 
		       "int EndID()");
    Cylinder_class.def("Label", &Cylinder::Label, 
		       "string	Label()");
    Cylinder_class.def("Print", &Cylinder::print,
		       print_overloads("Print(int id = -1, FILE out = stdout)"));

    Cylinder_class.def("toInventor", &Cylinder::toInventor,
		       Cylinder_toInventor("toInventor(FILE out = stdout, id = 0, indent = 1, with_head = false)"));
    Cylinder_class.def("toDejaVu", &Cylinder::toDejaVu,
		       Cylinder_toDejaVu("toDejaVu(FILE out = stdout, id = 0)"));

    Cylinder_class.def("toCosec", &Cylinder::toCosec,
		       Cylinder_toCosec("toCosec(FILE out = stdout, id = 0)"));

    Cylinder_class.def("setCenter", &Cylinder::setCenter, 
		       "setCenter(float x, float y, float z)");
    Cylinder_class.def("setAxis", (void (Cylinder::*)(const Vect&)) &Cylinder::setAxis, 
		       "setAxis(Vect axisdir)\nsetAxis(Euler axisdir)");
    Cylinder_class.def("setAxis", (void (Cylinder::*)(Euler&))&Cylinder::setAxis, 
		       "setAxis(Vect axisdir)\nsetAxis(Euler axisdir)");
    Cylinder_class.def("setRadius", &Cylinder::setRadius, 
		       "setRadius(float r)");
    Cylinder_class.def("setLength", &Cylinder::setLength, 
		       "setLength(float l)");
    Cylinder_class.def("setWeight", &Cylinder::setWeight, 
		       "setWeight(float w)");
    Cylinder_class.def("setStartID", &Cylinder::setStartID, 
		       "setStartID(int i)");
    Cylinder_class.def("setEndID", &Cylinder::setEndID, 
		       "setEndID(int i)");
    Cylinder_class.def("setLabel", &Cylinder::setLabel, 
		       "setLabel(string l)");
    Cylinder_class.def("PlaneAngleDiff", &Cylinder::PlaneAngleDiff, 
		       "float PlaneAngleDiff(Cylinder next)\n  dihedral angle of planes formed by the two axes and center connection vector");
    Cylinder_class.def("AxisAngleDiff", &Cylinder::AxisAngleDiff, 
		       "float AxisAngleDiff(Cylinder next)\n  difference in orientation (axises)");
    Cylinder_class.def("Angle2Vect", &Cylinder::Angle2Vect, 
		       "float Angle2Vect(Vect v)\n  difference in orientation axis of this and the direction vector ");
    Cylinder_class.def("Angle2Point", &Cylinder::Angle2Point, 
		       "float Angle2Point(Vect p)\n  difference in orientation between line passing both P and this center and axis of this");
    Cylinder_class.def("Angle2Center", &Cylinder::Angle2Center, 
		       "float Angle2Center(Cylinder next)\n  difference in orientation between line passing both centers and axis of this");
    Cylinder_class.def("NormDistance", &Cylinder::NormDistance, 
		       "float NormDistance(Cylinder next)\n  shortest distance of the two axes ");
    Cylinder_class.def("CenterDistance", &Cylinder::CenterDistance, 
		       "float	CenterDistance(Cylinder next)\n  distance of the center of next cylinder to the center of this");
    Cylinder_class.def("Distance2Point", &Cylinder::Distance2Point, 
		       "float	Distance2Point(x, y, z)\n  distance of the Point to the center of this cylinder");
    Cylinder_class.def("if_merge", &Cylinder::if_merge,
		       Cylinder_if_merge("bool if_merge(Cylinder next, da_limit = 30*PI/180., distance_limit = 3.2)\n  if next is ok to merge with this"));

    Cylinder_class.def("merge", &Cylinder::merge, 
		       "bool merge(Cylinder next)\n  actually merge both into this");
    Cylinder_class.def("mirror", &Cylinder::mirror, 
		       "mirror()\n  mirror the cylinders");
    Cylinder_class.def("Transform", (void (Cylinder::*)(Euler&, Vect&, Vect&))&Cylinder::Transform, 
		       "Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");
    Cylinder_class.def("Transform", (void (Cylinder::*)(Quater&, Vect&, Vect&))&Cylinder::Transform, 
		       "Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");


}
