#!/bin/env python
# this program simulates the 'einstein from noise' problem
# it will add noise to a base image n times. The base image
# is randomly translated by up to <dxmax> pixels. Each image is
# aligned to ref and averaged together. Outputs are generated
# at #particles specified by svpt
# <ref> <base> <basecoef> <dxmax> <# ptcl 2 gen> [posonly] [saveraw] [label=] [verbose]

from EMAN import *
from sys import argv,stdout
from math import floor
import random

ref=EMData()
ref.readImage(argv[1],0)

base=EMData()
base.readImage(argv[2],0)

basecoef=float(argv[3])

dxmax=int(argv[4])

n=int(argv[5])
svpt=[250,1000,2000,4000,8000,16000,32000,64000]

posonly=0
saveraw=0
label=""
verbose=0
for i in argv[6:]:
	j=i.split('=')
	if i=="posonly" : posonly=1
	elif i=="saveraw" : saveraw=1
	elif i=="verbose" : verbose=1
	elif j[0]=="label" : label=j[1]
	else : print "unknown option ",i

avg=ref.copy(0,0)
avg.zero()
bad=0
for i in range(1,n+1):
	if (not verbose and i%100==0) : 
		print "    %d\r"%i,
		stdout.flush()
	a=base.copy(0,0)
	a.zero(1.0)
	base2=base.copy(0,0)
	base2*=basecoef
	if (dxmax>0) :
		base2.setTAlign(random.randint(-dxmax,dxmax),random.randint(-dxmax,dxmax),0)
		base2.rotateAndTranslate()
	a+=base2
	a.edgeNormalize()
	if (i<=10 and saveraw) : a.writeImage("noisy"+label+".hdf",-1)

	a.transAlign(ref,-1,0,int(floor(dxmax*1.5+1)))
	if verbose : print i,")",a.Dx(),"\t",a.Dy(),"\t",
	a.rotateAndTranslate()
#	a.writeImage("rand.hed",-1)
	d=a.dot(ref)
	if verbose : print d
	if (d<0) : bad+=1
	else : avg+=a
	if (i in svpt) :
		sv=avg.copy(0,0)
		sv.normalize()
		sv.setNImg(i)
		sv.writeImage("avg"+label+".hdf",-1)

if (not i in svpt) :
	sv=avg.copy(0,0)
	sv.normalize()
	sv.setNImg(i)
	sv.writeImage("avg"+label+".hdf",-1)

print bad," were negative"
