// applyctf.C  2/9/02  Steve Ludtke

//N applyctf
//F Applies/corrects CTF to sets of particle images
//T LM
//1
//P <input file>	Source file
//P <output file>	Destination file, output overwrites existing images
//P [setparm]	Includes the CTF parameters in the header in the output images
//P [fixnoise]	n4 -> 0
//P [parmimg=<input>]	Extract CTF parameters from the first image in <input>
//P [parmmem=<name>]	Extract CTF parameters from ctfparm.txt with label <name>
//P [parm=<parmlist>]	Specify comma separated CTF parameters as specified in ctfparm.txt
//P [sffile=<file>]	File containing the structure factor associated with the fit parameters. Overrides value already specified.
//P [applyctf]	Applies absolute value of CTF as filter
//P [flipphase]	Flips phases 
//P [invert]	Inverts the densities (ie - mult by -1)
//P [applyB[=<B factor>]]	Applies envelope function, optionally specify override for B factor
//P [addnoise[=<scale>]]	Adds defined noise level to image. Optional <scale> scales overall noise amplitude.
//P [wiener]	Apply a Wiener filter
//P [mrc]	Output image(s) in MRC format
//P [inplace]	Ouput image numbers same as input numbers, otherwise appends
//U applyctf proj.hed proj.ctf.hed parmimg=start.hed applyctf
//D This program is used to apply or partially correct for the CTF in a set
//E of particle images. Parameters may come from several sources, and each part
//E of the correction may be individually enabled.
//D To fairly realistically simulate a micrograph use 'applyctf flipphase addnoise'. This
//E assumes that the images are already filtered to an appropriate resolution.
//E Note that the scale of added noise may be off and need adjustment.

//#define DEBUG0

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include "XYData.h"

main(int argc, char *argv[]) {
char *parmimage=NULL,*parmmem=NULL;
float parm[11] = { -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0 };
char *sffile=NULL;
int applyctf=0,flipphase=0,applyb=0,addnoise=0,wiener=0,mrcout=0,inplace=0,setparm=0,quiet=0,invert=0;
int fixnoise=0;
int i;

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

for (i=3; i<argc; i++) {
	if (strncmp(argv[i],"parmimg=",8)==0) {
		EMData tmp;
		tmp.readImage(argv[i]+8,0);
		if (!tmp.hasCTF()) error(ERR_EXIT,"No CTF parms in parmimg");
		memcpy(parm,tmp.getCTF(),11*sizeof(float));
	}
	else if (strncmp(argv[i],"parmmem=",8)==0) {
		FILE *in=fopen("ctfparm.txt","r");
		char s[256],t[200];
		if (!in) error(ERR_EXIT,"Cannot open ctfparm.txt");
		sffile=(char*)malloc(128);
		while (fgets(s,255, in)) {
			sscanf(s,"%s %f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%s",t,parm,parm+1,parm+2,parm+3,
				parm+4,parm+5,parm+6,parm+7,parm+8,parm+9,parm+10,sffile);
			if (strcmp(t,argv[i]+8)==0) break;
		}
		if (strlen(s)<2) error(ERR_EXIT,"Named memory not found in ctfparm.txt");
		fclose(in);
	}
	else if (strncmp(argv[i],"parm=",5)==0) {
		sffile=(char *)malloc(200);
		sffile[0]=0;
		sscanf(argv[i]+5,"%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%s",parm,parm+1,parm+2,parm+3,
				parm+4,parm+5,parm+6,parm+7,parm+8,parm+9,parm+10,sffile);
	}
	else if (strncmp(argv[i],"sffile=",7)==0) {
		sffile=argv[i]+7;
	}
	else if (strncmp(argv[i],"inplace",7)==0) inplace=1;
	else if (strncmp(argv[i],"quiet",5)==0) quiet=1;
	else if (strncmp(argv[i],"applyB",6)==0) {
		applyb=1;
		if (argv[i][6]=='=') parm[1]=atof(argv[i]+7);
	}
	else if (strncmp(argv[i],"addnoise",8)==0) {
		addnoise=1;
		if (argv[i][8]=='=') parm[6]*=atof(argv[i]+9);
	}
	else if (strncmp(argv[i],"applyctf",8)==0) applyctf=1;
	else if (strncmp(argv[i],"fixnoise",8)==0) fixnoise=1;
	else if (strncmp(argv[i],"invert",6)==0) invert=1;
	else if (strncmp(argv[i],"setparm",8)==0) setparm=1;
	else if (strncmp(argv[i],"flipphase",9)==0) flipphase=1;
	else if (strncmp(argv[i],"wiener",6)==0) wiener=1;
	else if (strncmp(argv[i],"usefs",5)==0) ;
	else if (strncmp(argv[i],"mrc",3)==0) mrcout=1;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}


// If no parameters are specified, check ctfparm.txt for a memory
// with the same name as the input file
if (parm[10]<=0) {
	printf("No CTF parameters specified, I'll check ctfparm.txt\n");
	FILE *in=fopen("ctfparm.txt","r");
	char s[256],t[200];
	if (!in) error(ERR_EXIT,"Cannot open ctfparm.txt");
	sffile=(char*)malloc(128);
	while (fgets(s,255, in)) {
		sscanf(s,"%s %f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%s",t,parm,parm+1,parm+2,parm+3,
			parm+4,parm+5,parm+6,parm+7,parm+8,parm+9,parm+10,sffile);
		int cl=strlen(t);
		if (strlen(argv[1])<cl) cl=strlen(argv[1]);
		if (strncmp(t,argv[1],cl)==0) break;
	}
	if (strlen(s)<2) error(ERR_EXIT,"Named memory not found in ctfparm.txt");
	fclose(in);
}

if (wiener) error(ERR_EXIT,"Wiener filter not supported yet");
if (addnoise) error(ERR_WARN,"Addnoise is flaky in the current version");
int n=fileCount(argv[1]);
if (n<1) error(ERR_EXIT,"No images to process");

int Ref;
if (!quiet) {
	Ref=LOGbegin(argc,argv);
	LOG(Ref,argv[1],LOG_INFILE,NULL);
	LOG(Ref,argv[2],LOG_OUTFILE,NULL);
}

EMData test;
test.readImage(argv[1],0,1);

if (fixnoise) parm[4]=0;
EMData *filter = new EMData();
filter->setCTF(parm);
filter->setSize(test.xSize()+2,test.ySize(),1);
filter->setComplex(1);
filter->setRI(1);
filter->ctfMap(applyctf+flipphase*2+applyb*4);
#ifdef DEBUG0
filter->writeMRC("filt.mrc");
#endif

EMData *noise = new EMData();
noise->setSize(test.xSize()+2,test.ySize(),1);
noise->setComplex(1);
noise->setRI(1);
noise->setCTF(parm);
noise->ctfMap(0);

noise->writeMRC("noise.mrc");

// process the images
for (i=0; i<n; i++) {
	EMData orig;
	orig.readImage(argv[1],i);
	EMData *fft=orig.doFFT();
	if (applyctf || flipphase || applyb) fft->mult(filter);
	if (addnoise) {
		float *d = fft->getData();
		float *n = noise->getDataRO();
		for (int j=0; j<fft->xSize()*fft->ySize(); j+=2) {
			float f=grand(n[j],n[j]/3);
			float a=frand(0,2.0*PI);
			d[j]+=f*cos(a);
			d[j+1]+=f*sin(a);
		}
		fft->doneData();
		noise->doneData();
		fft->writeMRC("done.mrc");
	}
	EMData *out=fft->doIFT();
	if (setparm) out->setCTF(parm);
	if (invert) out->multConst(-1.0);
	if (mrcout) {
		if (n==1) out->writeMRC(argv[2]);
		else {
			char s[1024];
			strcpy(s,argv[2]);
			if (s[strlen(s)-4]=='.') {
				s[strlen(s)-4]=0;
				sprintf(s+strlen(s),".%03d%s",i,argv[2]+strlen(argv[2])-4);
			}
			else sprintf(s+strlen(s),".%03d",i);
			out->writeMRC(s);
		}
	}
	else {
		if (inplace) out->writeImage(argv[2],i);
		else out->writeImage(argv[2],-1);
	}
	fft->gimmeFFT();
	delete out;
}

if (!quiet) LOG(Ref,NULL,LOG_END,NULL);
}
