// classalignall.C 1998 - Steve Ludtke
// Copyright 1998 Baylor College of Medicine

//N classalignall
//F This will align the images in all of the cls*img files in the current directory
//T LA
//1
//P <iterations>	Number of loops
//P []	See classalign2 for options
//D This program will align the images in each cls*img file to generate a
//E class average. It is simply a wrapper for <a href=classalign2.html>classalign2</a>.
//E All of the options are documented there. The &lt;input file> parameter
//E is excluded in this case, since this program automatically operates on all cls files.


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <ctype.h>
#ifndef AIX
#include <sys/wait.h>
#endif
#include "util.h"
#include "EMData.h"

main(int argc, char *argv[]) {
int i,ign1=0,slow=0;
int iter=2,norm,r1=0,nosum=0,badf=0,stat=0,sr=0,mask=-1,median=0,sal=0,cls=0,can=2,ml=0,refmask=0,refine=0,phase=0;
int nproc=0,minproc=0,maxproc=0,first=0,dot=0,precen=0,imask=-1,half=0,sigfilt=0,sigim=0,sali=0,maxshift=-1;
int usefilt=0,dfilt=0;
float keep=1.2,ctfc=0;
FILE *out;
const char *tmp;
char *ctfcw=NULL,*logit=NULL;
char s[512],t[600];

appinit(argc,argv);

if (argc<2) { appusage(argv[0]); exit(0); }


for (i=1; i<argc; i++) {
	if (strncmp("ref1",argv[i],4)==0) r1=1;
	else if (strncmp("ign1",argv[i],4)==0) ign1=1;
	else if (strncmp("dfilt",argv[i],5)==0) dfilt=1;
	else if (strncmp("finalref",argv[i],8)==0) r1=2;
	else if (strncmp("nosum",argv[i],5)==0) nosum=1;
	else if (strncmp("badfile",argv[i],7)==0) badf=1;
	else if (strncmp("sigfilt",argv[i],7)==0) sigfilt=1;
	else if (strncmp("refmaskali",argv[i],10)==0) refmask=2;
	else if (strncmp("refmask",argv[i],7)==0) refmask=1;
	else if (strncmp("phase",argv[i],5)==0) phase=1;
	else if (strncmp("fsc",argv[i],3)==0) phase=2;
	else if (strncmp("saverefs",argv[i],8)==0) sr=1;
	else if (strncmp("savecen",argv[i],7)==0) sal=1;
	else if (strncmp("saveali",argv[i],7)==0) sali=1;
	else if (strncmp("imask=",argv[i],6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp("stat",argv[i],4)==0) stat=1;
	else if (strncmp("slow",argv[i],4)==0) slow=1;
	else if (strncmp("precen",argv[i],6)==0) precen=1;
	else if (strncmp("ctfc=",argv[i],5)==0) ctfc=atof(&argv[i][5]);
	else if (strncmp("ctfcw=",argv[i],6)==0) ctfcw=&argv[i][6];
	else if (strncmp("logit=",argv[i],6)==0) logit=&argv[i][6];
	else if (strncmp("cls=",argv[i],4)==0) cls=atoi(&argv[i][4]);
	else if (strncmp("dot",argv[i],3)==0) dot=1;
	else if (strncmp("median",argv[i],6)==0) median=1;
	else if (strncmp("proc=",argv[i],5)==0) sscanf(&argv[i][5],"%d,%d",&minproc,&maxproc);
	else if (strncmp("keep=",argv[i],5)==0) keep=atof(&argv[i][5]);
	else if (strncmp("mask=",argv[i],5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp("maxshift=",argv[i],9)==0) maxshift=atoi(&argv[i][9]);
	else if (strncmp("first=",argv[i],6)==0) first=atoi(&argv[i][6]);
	else if (strncmp("refine",argv[i],6)==0) refine=1;
	else if (strncmp("even",argv[i],4)==0) half=2;
	else if (strncmp("odd",argv[i],3)==0) half=1;
	else if (strncmp("ca3",argv[i],3)==0) can=3;
	else if (strncmp("usefilt",argv[i],7)==0) usefilt=1;
	else if (strncmp("ml",argv[i],2)==0) ml=1;
	else if (strncmp("sigmaimg",argv[i],8)==0) sigim=1;
	else {
		if (argv[i][0]>='0' && argv[i][0]<='9') iter=atoi(argv[i]);
		else { printf("Unknown argument '%s'\n",argv[i]); exit(1); }
	}
}

if (minproc<=0) minproc=1;
if (maxproc<=minproc) maxproc=minproc;

int Ref=LOGbegin(argc,argv);

unlink("classes.hed");
unlink("classes.img");

tmp=tmpnam2();
out=fopen(tmp,"w");
if (!out) { printf("Cannot create temp. file\n"); exit(1); }

for (i=first; i<10000; i++) {
	if (cls) sprintf(s,"cls%1d.%04d.lst",cls,i); 
	else sprintf(s,"cls%04d.lst",i);
	if (access(s,R_OK)) break;

	if (ml) {
		sprintf(t,"classalignml %s %d %s %s %s mask=%d %s locfromfile",
			s,iter,r1==1?"ref1":(r1==2?"finalref":""),sr?"saverefs":"",median?"median":"",mask,
			half==0?"":(half==1?"odd ":"even "));
		if (ign1) sprintf(t+strlen(t)," ign1");
		if (ctfc!=0) sprintf(t+strlen(t)," ctfc=%f",ctfc);
		if (ctfcw) sprintf(t+strlen(t)," ctfcw=%s",ctfcw);
		if (i!=0) sprintf(t+strlen(t)," quiet");
		fprintf(out,"%s\n",t);
	}
	else if (iter<=0) {
		sprintf(t,"ctfavg2 %s classes.hed keepsig=%f fileali copyeuler %s %s %s",s,keep,sr?"copyrefs":"",refmask==1?"refmask ":" ",r1?"ref0":"");
		if (ctfcw) sprintf(t+strlen(t)," ctfcw=%s",ctfcw);
		else if (ctfc>0) sprintf(t+strlen(t)," ctfc=%1.2f",ctfc);
		if (half==1) sprintf(t+strlen(t)," odd");
		if (half==2) sprintf(t+strlen(t)," even");
		if (i!=0) sprintf(t+strlen(t)," quiet");
		fprintf(out,"%s\n",t);
	}
	else {
		sprintf(t,"classalign%0d %s %s %s %s %s %s %s %s %s %s %s%s keep=%f mask=%d imask=%d %d %s %s %s %s %s locfromfile",
			can,s,nosum?"nosum":"",stat?"stat":"",badf?"badfile":"",r1==1?"ref1":(r1==2?"finalref":""),
			sr?"saverefs":"",median?"median":"",sal?"savecen":"",sali?"saveali":"",sigfilt?"sigfilt":"",logit?"logit=":"",logit?logit:"",keep,mask,imask,iter,
			dot?"dot ":"",precen?"precen ":"",half==0?"":(half==1?"odd ":"even "),sigim?"sigmaimg ":"",refmask==1?"refmask ":(refmask==2?"refmaskali":""));
		if (ign1) sprintf(t+strlen(t)," ign1");
		if (phase==1) sprintf(t+strlen(t)," phase");
		if (phase==2) sprintf(t+strlen(t)," fsc");
		if (refine) sprintf(t+strlen(t)," refine");
		if (dfilt) sprintf(t+strlen(t)," dfilt");
		if (ctfc!=0) sprintf(t+strlen(t)," ctfc=%f",ctfc);
		if (ctfcw) sprintf(t+strlen(t)," ctfcw=%s",ctfcw);
		if (slow) strcat(t," slow");
		if (usefilt) strcat(t," usefilt");
		if (maxshift>=0) sprintf(t+strlen(t)," maxshift=%d",maxshift);
		if (i!=0) sprintf(t+strlen(t)," quiet");
		fprintf(out,"%s\n",t);
	}
}
fclose(out);

// This makes sure the classes.hed/img file starts out at the correct
// length. Otherwise there are problems when appending on some
// platforms despite file locking
EMData d1;
d1.readImage("cls0000.lst",0);
d1.zero();
d1.writeIMAGIC("classes.hed",sr?2*i-1:i-1);

sprintf(t,"runpar proc=%d,%d file=%s",minproc,maxproc,tmp);
run(t);

unlink(tmp);
//puts(tmp);
LOG(Ref,NULL,LOG_END,NULL);
}

// basically a version of 'system'
/*void run(char *s) {
pid_t f=fork();

if (!f) execl("/bin/sh","/bin/sh","-c",s,0);
//printf("%d exit\n",waitpid(f,NULL,0));
waitpid(f,NULL,0);

return;
}
*/
