// clean3d.C 1/00 Steve Ludtke
// Copyright 2000 Baylor College of Medicine

//N clean3d
//F This program uses make3diter and proc3d to iteratively make models match class averages
//T LR
//1
//P <classes>	File with class averages
//P <3d model>	Preliminary 3d model to clean up
//P [sym=<sym>]	Symmetry specification
//P [proc=<nproc>]	Number of processors to use
//P [3dit=<n iter>]	Number of 'normal' cleaning iterations
//P [3dit2=<n iter>]	Number of 'zeroing' cleaning iterations
//P [keepthr=<thr>]	Threshold for exclusion of class-averages (see make3diter)
//P [keepsig=<sigmult>]	Alternate specification of threshold (see make3diter)
//P [speed=<1-5>]	Speed in make3diter, convergence rate, dfl=1
//P [fixamp]	Uses the Fourier amplitudes from the original image
//U clean3d classes.5.img threed.5a.img sym=d7 3dit=2 3dit2=2
//D This program performs iterative real-space refinement to make
//E a 3D model more closely match a set of class-averages. There are 2
//E versions of the routine. The '3dit=' routine simply tries for the
//E 'median' match. The '3dit2=' routine drives the densities towards zero
//E as much as is permitted by the class averages.
//D The algorithm used by this program is quite difficult to describe.
//E The basic idea is to calculate the differences between each class average
//E and a corresponding projection of the model, then compute a 3D 'error model'
//E describing the error in each pixel compared to unfiltered back-projection.
//E However, this isn't strictly accurate, since not just the average error value
//E is calculated for each voxel. In reality, we calculate a statistical set of
//E predicted errors for each voxel. '3dit' uses the median value of the
//E predicted error. '3dit2' uses a value between the median value, and the
//E max/min value which would move the density towards zero. For a real explanation,
//E you'll have to dissect the code. We'd like to retire this algorithm and replace
//E it with something more mathematically robust, but in testing so far, it has
//E produced the best corrections.
//D The '3dit2' algorithm can get a bit carried away if you run it for too many iterations.
//E From experience we suggest '3dit2=2 3dit=1' will generally provide good results. Using
//E the 'automask2' option of <a href=proc3d.html>proc3d</a> is also a good idea. during
//E iterative refinement.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int i,j;
char *sym=NULL;
int it1=0,it2=0,proc=1,fixamp=0;
float keepsig=-9999.0,keepthr=0,speed=1.0;
char s[512];
const char *t = tmpnam2();
FILE *out;

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

for (i=3; i<argc; i++) {
	if (strncmp("3dit=",argv[i],5)==0) it1=atoi(&argv[i][5]);
	else if (strncmp("3dit2=",argv[i],6)==0) it2=atoi(&argv[i][6]);
	else if (strncmp("usefs=",argv[i],6)==0) ;
	else if (strncmp("proc=",argv[i],5)==0) proc=atoi(&argv[i][5]);
	else if (strncmp("speed=",argv[i],6)==0) speed=atof(&argv[i][6]);
	else if (strncmp("fixamp",argv[i],6)==0) fixamp=1;
	else if (strncmp("keepthr=",argv[i],8)==0) keepthr=atof(&argv[i][8]);
	else if (strncmp("keepsig=",argv[i],8)==0) keepsig=atof(&argv[i][8]);
	else if (strncmp(argv[i],"sym=",4)==0) {
        sym=&argv[i][4];
		Euler e(0,0,0);
		e.setSym(sym);
        if (e.getMaxSymEl()==0) { printf("Unsupported symmetry\n"); exit(1); }
	}
	else { printf("Unknown argument '%s'\n",argv[i]); exit(1); }
}

if (proc>1) { error(ERR_WARN,"Multiple processors currently disabled in clean3d (3dit)"); proc=1; }

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_REFFILE,NULL);
LOG(Ref,argv[2],LOG_INFILE,NULL);


if (!sym) sym="i";

if (fixamp) {
	sprintf(s,"cp %s tmp.mrc",argv[2]);
	run(s);
}

for (i=0; i<it2; i++) {
	out=fopen(t,"w");
	for (j=proc-1; j>=0; j--) {
		fprintf(out,"make3diter %s %s %s frac=%d/%d speed=%f tozero",argv[2],argv[1],argv[2],j,proc,speed);
		if (keepthr) fprintf(out," keepthr=%f",keepthr);
		if (keepsig!=-9999.0) fprintf(out," keepsig=%f",keepsig);
		fprintf(out,"\n");
	}
	fclose(out);

	sprintf(s,"runpar proc=%d file=%s",proc,t);
//	sprintf(s,"cat %s",t);
	sleep(5);	// a hack to try to fix the rsh problem
	run(s);
	unlink(t);

	if (strlen(sym)>1) {
		sprintf(s,"proc3d %s %s sym=%s",argv[2],argv[2],sym);
		run(s);
	}
}

for (i=0; i<it1; i++) {
	out=fopen(t,"w");
	for (j=proc-1; j>=0; j--) {
//		fprintf(out,"make3diter %s %s %s sym=%s frac=%d/%d\n",argv[2],argv[1],argv[2],sym,j,proc);
		fprintf(out,"make3diter %s %s %s speed=%f frac=%d/%d",argv[2],argv[1],argv[2],speed,j,proc);
		if (keepthr) fprintf(out," keepthr=%f",keepthr);
		if (keepsig!=-9999.0) fprintf(out," keepsig=%f",keepsig);
		fprintf(out,"\n");
	}
	fclose(out);

	sprintf(s,"runpar proc=%d file=%s",proc,t);
//	sprintf(s,"cat %s",t);
	run(s);
	unlink(t);

	if (strlen(sym)>1) {
		sprintf(s,"proc3d %s %s sym=%s",argv[2],argv[2],sym);
		run(s);
	}
}

if (fixamp) {
	sprintf(s,"proc3d %s tmp.mrc pha1amp2=%s",argv[2],argv[2]);
	unlink("tmp.mrc");
}

LOG(Ref,argv[2],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);

}
