// cloud.C 1998 Phil Baldwin
// Copyright 1998 Baylor College of Medicine
/*              cloud.C 
    is a variant of pca.c. The acknowledgements for this latter program are
	 appended at the end of this one.
	 
	cloud.C takes 
	 			a  file of images
		and outputs another
		         file of images.
       ***********S****************
	      The output will show the features that are most different from
		   each other.
          ****************************
		  the user picks the number of factors = CUTOFF

*/

//N cloud
//F This program performs MSA based filtering of a set of particles
//T LT
//1
//P <input>	Images to be filtered
//P <output>	Filtered output
//P <elements>	Number of MSA components to include
//P [1 writes eigenims.img]	Flag to write eigenimages
//D This program is classified as experimental and is not documented yet.
//E This program will take a set of 2d aligned particles and perform MSA
//E analysis on them. The resulting MSA basis vectors (images) will then be
//E used as a new basis for the raw images. This is equivalent to a
//E filtering operation.

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include "math.h"
#define SIGN(a, b) ( (b) < 0 ? -fabs(a) : fabs(a) )
#define TESTFILE 0

void corcol(float **data,int n,int m,float **symmat);
void covcol(float **data,int n,int m,float **symmat);
void scpcol(float **data,int n,int m,float **symmat);
void erhand(char err_msg[]);
float *make_vector(int n);
float **matrix(int n,int m);
void free_vector(float *v,int n);
void free_matrix(float **mat,int n,int m);
void tred2(float **a,int n,float *d,float *e);
void tqli(float d[],float e[],int n,float **z);


main(int argc,char *argv[])
{
List *imagelist;
FILE *testfile;
EMData *dD;
int  n2, n,  i, j, k,  l, m, npart, isize;
int index, jmax; //PRB
int nmin, nmax, EIGENIMS;
int CUTOFF;
float **finaleigenvecs, *evals, *interm, tempij;
float **symmat, **symmat2,  projectionjl ;
float **data, **datacopy,**dataout, *imagedata; // remember imagedata is a simple array
float *column, evalmax; // PRB
float stddev;
char option, *strncpy();

appinit(argc,argv);


if (argc !=  4 && argc != 5) appusage(argv[0]);

imagelist = readImages(argv[1],0,-1);
npart     = imagelist -> count();

isize  = ((EMData *)imagelist->objectAt(0) ) -> xSize();
n2 = isize * isize;   // for msa.C and footprints, n2=32
CUTOFF = atoi(argv[3]);
EIGENIMS = atoi(argv[4]);

if (TESTFILE){ testfile = fopen("test","r"); }

printf("No. of rows: %d, no. of columns: %d, final n2: %d.\n",n2,npart,CUTOFF);
printf("Input file: %s.\n",argv[1]);
 
nmin = n2; if (npart < nmin) nmin = npart;
nmax = npart + n2-nmin;

printf("nmin: %d, nmax: %d \n",nmin,nmax);


m = npart;  // m is the  number of particles. Associate index j  .
n = n2;  //    n  is isize*isize = 6400     BIGGGG !!!!!!!!,  Associate index i.

if (TESTFILE){ n= 3; m=2;}
        
column = (float *) malloc((n)*sizeof(float));
 



// CUTOFF = 5;


printf("m: %d, n: %d, CUTOFF: %d.\n",m,n,CUTOFF);
 
    /* Now read in data. */

    data = matrix(n, m);  /* Storage allocation for input data , npart by m  */
	datacopy = matrix(n,m);
	finaleigenvecs = matrix(n,m);
	dataout = matrix(n,m);
	symmat = matrix(m, m);  /* Allocation of correlation (etc.) matrix */
	symmat2 = matrix(m, m);  /* Allocation of correlation (etc.) matrix */
    /* Allocate storage for dummy and new vectors. */
    evals = make_vector(m);     /* Storage alloc. for vector of eigenvalues */
    interm = make_vector(m);    /* Storage alloc. for 'intermediate' vector */
       
	for (j = 1; j <= m; j++)  {// j labels the particle
//		printf("now read dD at %d \n",j);
		dD = (EMData*)imagelist -> objectAt(j-1);
		imagedata = dD -> getData();
		
		for (i = 1; i <= n ; i++) { // i labels the pixel
            

			if (TESTFILE) { fscanf(testfile, "%f", &data[i][j]);
			} else {
 				data[i][j] = imagedata[i-1]; 
			} 
			datacopy[i][j] = data[i][j];} //ends i loop                
		dD -> doneData();
		} 
         

if (0) {
	printf("new footprint of particles 1, 2,3,4 is \n");
 	for (j = 1; j <= m; j++)  {// j labels the footprint
		stddev = 0.;
		for (i = 1; i <= n; i++)  // i labels the particle
			{ stddev = data[i][j]*data[i][j] ;
			 data[i][j]*=  sqrt(sqrt(1/stddev));			
			if (i<3) printf(" %f ",data[i][j]);       // info is stored in start.fp
			if (i==2) printf("\n"); }} // ends i, j loops
	}	
		   /* Check on (part of) input data.
    for (i = 1; i <= 18; i++) {for (j = 1; j <= 8; j++)  {
        printf("%7.1f", data[i][j]);  }  printf("\n");  } */
		
	printf("finished reading in data \n");

 
   /* Look at analysis option; branch in accordance with this. */
   option = 'S';
  /* Look at analysis option; branch in accordance with this. */

     switch(option)
             {
          case 'R':
          case 'r':
              printf("Analysis of correlations chosen.\n");
              corcol(data, n, m, symmat);

                          /* Output correlation matrix.
                          for (i = 1; i <= m; i++) {
                           for (j = 1; j <= 8; j++)  {
                            printf("%7.4f", symmat[i][j]);  }
                            printf("\n");  }
                          */
              break;
          case 'V':
          case 'v':
              printf("Analysis of variances-covariances chosen.\n");
              covcol(data, n, m, symmat);

                          /* Output variance-covariance matrix.
                          for (i = 1; i <= m; i++) {
                          for (j = 1; j <= 8; j++)  {
                            printf("%7.1f", symmat[i][j]);  }
                            printf("\n");  }
                          */
              break;
          case 'S':
          case 's':
              printf("Analysis of sums-of-squares-cross-products");
              printf(" matrix chosen.\n");
              scpcol(data, n, m, symmat);

                         /* Output SSCP matrix.
                         for (i = 1; i <= m; i++) {
                          for (j = 1; j <= 8; j++)  {
                            printf("%7.1f", symmat[i][j]);  }
                            printf("\n");  }
                         */
              break;
          default:
/*              printf("Option: %s\n",option); */
              printf("For option, please type R, V, or S\n");
              printf("(upper or lower case).\n");
              printf("Exiting to system.\n");
              exit(1);
              break;
          }

/*********************************************************************
    Eigen-reduction
**********************************************************************/

    for (i = 1; i <= m; i++) {
     for (j = 1; j <= m; j++) {
      symmat2[i][j] = symmat[i][j]; /* Needed below for col. projections */
                              }}
							  							  
    tred2(symmat, m, evals, interm);  /* Triangular decomposition */
	
//   for (i = 1; i <= m; i++) { for (j = 1; j <= m; j++) {
//      printf("symmat[%d][%d] = %f\n",i,j,symmat[i][j]); /* Needed below for col. projections */
//                              } }
							  
    tqli(evals, interm, m, symmat);   /* Reduction of sym. trid. matrix */
    /* evals now contains the eigenvalues,
       columns of symmat now contain the associated eigenvectors. */

//       We need to reorder the eigenvalues and symmat
// but not  data
	for (i =1; i <= m ; i++){ // switching labels
       evalmax = -100;
		for (j =i; j <= m ; j++){ //switching labels 
			if (evals[j] > evalmax) {
				evalmax = evals[j];
				jmax = j;
			}
		}	
// now switch i and jmax entries
		evals[jmax] = evals[i];
		evals[i]    = evalmax;
// now switch i and jmax rows of symmat
		for (index=1;index <= m;index ++){
			column[index]    = symmat[index][i];
			symmat[index][i] = symmat[index][jmax];
			symmat[index][jmax] = column[index];
		}
	}

     printf("\nEigenvalues:\n"); fflush(stdout); 
     for (j = 1; j <= m; j++) {
       printf("%18.5f\n", evals[j]); }
//     printf("\n(Eigenvalues should be strictly positive; limited\n");
//     printf("precision machine arithmetic may affect this.\n");
//     printf("Eigenvalues are often expressed as cumulative\n");
//     printf("percentages, representing the 'percentage variance\n");
//     printf("explained' by the associated axis or principal component.)\n");


     printf("\nEigenvectors:\n");
     printf("(First %d; their definition in terms of original vbes.)\n", CUTOFF);
//            The eigenvectors are written as columns
	for (j=1; j <=6 && j <= m; j++)  printf("Eigen %d:\t", j);  
	printf("\n");

	for (i = 1;  i <= 8; i++)  {
		for (j = 1; j<= 6 && j <= m ; j++) {
          printf("%12.4f", symmat[i][j]);  } // newline after all the j's
          printf("\n");  }

 
		  
	printf("\n n is %d, m is %d , CUTOFF is %d \n", n , m , CUTOFF);
	
// ****************************************** Fancy stuff *******************
	printf("started to do finaleigenvecs \n"); fflush(stdout);

	for (j=1; j<= m ; j++) {
		for (i=1; i<= n; i++){
		tempij = 0;
		    for (k=1;k<=m;k++){
				tempij += datacopy[i][k] * symmat[k][j];
			}
		finaleigenvecs[i][j] = tempij/(sqrt(evals[j]));
		}}
		
// Now project the pictures onto these eigenvecs: First create projection
	printf("finished finaleigenvecs \n"); fflush(stdout);

// first index shold be the fast changing index	

if (EIGENIMS){	
		unlink("eigenims.img"); unlink("eigenims.hed");	
		dD = new EMData;
		dD -> setSize(isize,isize,1);
	for (j = 1; j <= CUTOFF; j++) {
		imagedata = dD -> getData();
		for (i = 1; i <= n; i++)  {
			imagedata[i-1]=finaleigenvecs[i][j] ;
		}   //    ends i loop 
		dD -> doneData();
		dD -> writeIMAGIC("eigenims.img",-1);
		}
		delete dD;
		printf("finished writing out eigenimages \n"); fflush(stdout);
}   //          ends        EIGENIMS	
		

	for (j=1; j<= m ; j++) {
		for (i=1; i<= n; i++){
			dataout[i][j]= 0;}}
		
//		printf("finished particle %d \r ",j);fflush(stdout);


	for (l=1; l <= CUTOFF ; l++)  {  // sum over eigenimages
		for (j=1; j<= m ; j++) {    // for each particle
			projectionjl =0;
			for (k=1;k<=n;k++){
				projectionjl += finaleigenvecs[k][l] * datacopy[k][j]; // amount
				                                  // of jth picture on lth eigenimage
			} // ends k loop
			for (i=1; i<= n; i++){ // contribution to final picture from ith eigenimage
				dataout[i][j] += finaleigenvecs[i][l] * projectionjl;
			} // ends i loop
		}	 // ends j   loop 
	}	 // ends l   loop 
		

	if (TESTFILE) {	
		for (i=1; i<= n; i++){
			for (j=1; j<= m ; j++){
				printf(" dataout[%d][%d] = %f \n", i, j, dataout[i][j]);
				printf(" datacopy[%d][%d] = %f \n", i, j, datacopy[i][j]);
				printf(" finalvecs[%d][%d] = %f \n", i, j, finaleigenvecs[i][j]);
				if (0) printf(" symmat[%d][%d] = %f \n", i, j, symmat[i][j]);
		}	} // ends i, j   loop 
	} // ends TESTFILE if loop
		


			
		
	printf("finished dataout \n"); fflush(stdout);

//	for (j=1; j <=6 && j <= m; j++)  printf("Particle  %d:\t", j);  
//	printf("\n");
	
	for (j = 1; j <= m; j++) {
		dD=(EMData *)imagelist->objectAt(j-1);
		imagedata = dD -> getData();
		for (i = 1; i <= n; i++)  {
		
//			q= dataout[i][m+1-j] ;
			imagedata[i-1]=dataout[i][j] ;
		}   //      printf("\n");  
		dD -> doneData();
		dD -> writeIMAGIC(argv[2],-1);
		}



    free_matrix(data, n, m);
    free_matrix(datacopy, n, m);
    free_matrix(dataout, n, m);
    free_matrix(finaleigenvecs, m, n);
    free_matrix(symmat, m, m);
    free_matrix(symmat2, m, m);
    free_vector(evals, m);
    free_vector(interm, m);

}

/**  Correlation matrix: creation  ***********************************/

void corcol(float **data,int n,int m,float **symmat)
/* Create m * m correlation matrix from given n * m data matrix. */
{
float eps = 0.005;
float x, *mean, *stddev;
int i, j, j1, j2;

/* Allocate storage for mean and std. dev. vectors */

mean = make_vector(m);
stddev = make_vector(m);

/* Determine mean of column vectors of input data matrix */

for (j = 1; j <= m; j++)
    {
    mean[j] = 0.0;
    for (i = 1; i <= n; i++)
        {
        mean[j] += data[i][j];
        }
    mean[j] /= (float)n;
    }

printf("\nMeans of column vectors:\n");
for (j = 1; j <= m; j++)  {
    printf("%7.1f",mean[j]);  }   printf("\n");

/* Determine standard deviations of column vectors of data matrix. */

for (j = 1; j <= m; j++)
    {
    stddev[j] = 0.0;
    for (i = 1; i <= n; i++)
        {
        stddev[j] += (   ( data[i][j] - mean[j] ) *
                         ( data[i][j] - mean[j] )  );
        }
        stddev[j] /= (float)n;
        stddev[j] = sqrt(stddev[j]);
        /* The following in an inelegant but usual way to handle
        near-zero std. dev. values, which below would cause a zero-
        divide. */
        if (stddev[j] <= eps) stddev[j] = 1.0;
    }

printf("\nStandard deviations of columns:\n");
for (j = 1; j <= m; j++) { printf("%7.1f", stddev[j]); }
printf("\n");

/* Center and reduce the column vectors. */

for (i = 1; i <= n; i++)
    {
    for (j = 1; j <= m; j++)
        {
        data[i][j] -= mean[j];
        x = sqrt((float)n);
        x *= stddev[j];
        data[i][j] /= x;
        }
    }

/* Calculate the m * m correlation matrix. */
for (j1 = 1; j1 <= m-1; j1++)
    {
    symmat[j1][j1] = 1.0;
    for (j2 = j1+1; j2 <= m; j2++)
        {
        symmat[j1][j2] = 0.0;
        for (i = 1; i <= n; i++)
            {
            symmat[j1][j2] += ( data[i][j1] * data[i][j2]);
            }
        symmat[j2][j1] = symmat[j1][j2];
        }
    }
    symmat[m][m] = 1.0;

return;

}

/**  Variance-covariance matrix: creation  *****************************/

void covcol(float **data,int n,int m,float **symmat)
/* Create m * m covariance matrix from given n * m data matrix. */
{
float *mean;
int i, j, j1, j2;

/* Allocate storage for mean vector */

mean = make_vector(m);

/* Determine mean of column vectors of input data matrix */

for (j = 1; j <= m; j++)
    {
    mean[j] = 0.0;
    for (i = 1; i <= n; i++)
        {
        mean[j] += data[i][j];
        }
    mean[j] /= (float)n;
    }

printf("\nMeans of column vectors:\n");
for (j = 1; j <= m; j++)  {
    printf("%7.1f",mean[j]);  }   printf("\n");

/* Center the column vectors. */

for (i = 1; i <= n; i++)
    {
    for (j = 1; j <= m; j++)
        {
        data[i][j] -= mean[j];
        }
    }

/* Calculate the m * m covariance matrix. */
for (j1 = 1; j1 <= m; j1++)
    {
    for (j2 = j1; j2 <= m; j2++)
        {
        symmat[j1][j2] = 0.0;
        for (i = 1; i <= n; i++)
            {
            symmat[j1][j2] += data[i][j1] * data[i][j2];
            }
        symmat[j2][j1] = symmat[j1][j2];
        }
    }

return;

}

/**  Sums-of-squares-and-cross-products matrix: creation  **************/

void scpcol(float **data,int n,int m,float **symmat)
/* Create m * m sums-of-cross-products matrix from n * m data matrix. */
{
int i, j1, j2;

/* Calculate the m * m sums-of-squares-and-cross-products matrix. */

for (j1 = 1; j1 <= m; j1++)
    {
    for (j2 = j1; j2 <= m; j2++)
        {
        symmat[j1][j2] = 0.0;
        for (i = 1; i <= n; i++)
            {
            symmat[j1][j2] += data[i][j1] * data[i][j2];
            }
        symmat[j2][j1] = symmat[j1][j2];
        }
    }

return;

}

/**  Error handler  **************************************************/

void erhand(char err_msg[])
/* Error handler */
{
    fprintf(stderr,"Run-time error:\n");
    fprintf(stderr,"%s\n", err_msg);
    fprintf(stderr,"Exiting to system.\n");
    exit(1);
}

/**  Allocation of vector storage  ***********************************/

float *make_vector(int n)
/* Allocates a float vector with range [1..n]. */
{

    float *v;

    v = (float *) malloc ((unsigned) n*sizeof(float));
    if (!v) erhand("Allocation failure in vector().");
    return v-1;

}

/**  Allocation of float matrix storage  *****************************/

float **matrix(int n,int m)
/* Allocate a float matrix with range [1..n][1..m]. */
{
    int i;
    float **mat;

    /* Allocate pointers to rows. */
    mat = (float **) malloc((unsigned) (n)*sizeof(float*));
    if (!mat) erhand("Allocation failure 1 in matrix().");
    mat -= 1;

    /* Allocate rows and set pointers to them. */
    for (i = 1; i <= n; i++)
        {
        mat[i] = (float *) malloc((unsigned) (m)*sizeof(float));
        if (!mat[i]) erhand("Allocation failure 2 in matrix().");
        mat[i] -= 1;
        }

     /* Return pointer to array of pointers to rows. */
     return mat;

}

/**  Deallocate vector storage  *********************************/

void free_vector(float *v,int n)
/* Free a float vector allocated by vector(). */
{
   free((char*) (v+1));
}

/**  Deallocate float matrix storage  ***************************/

void free_matrix(float **mat,int n,int m)
/* Free a float matrix allocated by matrix(). */
{
   int i;

   for (i = n; i >= 1; i--)
       {
       free ((char*) (mat[i]+1));
       }
   free ((char*) (mat+1));
}

/**  Reduce a real, symmetric matrix to a symmetric, tridiag. matrix. */

void tred2(float **a,int n,float *d,float *e)
/* Householder reduction of matrix a to tridiagonal form.
   Algorithm: Martin et al., Num. Math. 11, 181-195, 1968.
   Ref: Smith et al., Matrix Eigensystem Routines -- EISPACK Guide
        Springer-Verlag, 1976, pp. 489-494.
        W H Press et al., Numerical Recipes in C, Cambridge U P,
        1988, pp. 373-374.  */
{
int l, k, j, i;
float scale, hh, h, g, f;

for (i = n; i >= 2; i--)
    {
    l = i - 1;
    h = scale = 0.0;
    if (l > 1)
       {
       for (k = 1; k <= l; k++)
           scale += fabs(a[i][k]);
       if (scale == 0.0)
          e[i] = a[i][l];
       else
          {
          for (k = 1; k <= l; k++)
              {
              a[i][k] /= scale;
              h += a[i][k] * a[i][k];
              }
          f = a[i][l];
          g = f>0 ? -sqrt(h) : sqrt(h);
          e[i] = scale * g;
          h -= f * g;
          a[i][l] = f - g;
          f = 0.0;
          for (j = 1; j <= l; j++)
              {
              a[j][i] = a[i][j]/h;
              g = 0.0;
              for (k = 1; k <= j; k++)
                  g += a[j][k] * a[i][k];
              for (k = j+1; k <= l; k++)
                  g += a[k][j] * a[i][k];
              e[j] = g / h;
              f += e[j] * a[i][j];
              }
          hh = f / (h + h);
          for (j = 1; j <= l; j++)
              {
              f = a[i][j];
              e[j] = g = e[j] - hh * f;
              for (k = 1; k <= j; k++)
                  a[j][k] -= (f * e[k] + g * a[i][k]);
              }
         }
    }
    else
        e[i] = a[i][l];
    d[i] = h;
    }
d[1] = 0.0;
e[1] = 0.0;
for (i = 1; i <= n; i++)
    {
    l = i - 1;
    if (d[i])
       {
       for (j = 1; j <= l; j++)
           {
           g = 0.0;
           for (k = 1; k <= l; k++)
               g += a[i][k] * a[k][j];
           for (k = 1; k <= l; k++)
               a[k][j] -= g * a[k][i];
           }
       }
       d[i] = a[i][i];
       a[i][i] = 1.0;
       for (j = 1; j <= l; j++)
           a[j][i] = a[i][j] = 0.0;
    }
}

/**  Tridiagonal QL algorithm -- Implicit  **********************/

void tqli(float d[],float e[],int n,float **z)
{
int m, l, iter, i, k;
float s, r, p, g, f, dd, c, b;

for (i = 2; i <= n; i++)
    e[i-1] = e[i];
e[n] = 0.0;
for (l = 1; l <= n; l++)
    {
    iter = 0;
    do
      {
      for (m = l; m <= n-1; m++)
          {
          dd = fabs(d[m]) + fabs(d[m+1]);
//          if (fabs(e[m]) + dd == dd) break;
          if (fabs(e[m]) /dd <= 1.e-6) break;
          }
          if (m != l)
             {
             if (iter++ == 30) erhand("No convergence in TLQI.");
             g = (d[l+1] - d[l]) / (2.0 * e[l]);
             r = sqrt((g * g) + 1.0);
             g = d[m] - d[l] + e[l] / (g + SIGN(r, g));
             s = c = 1.0;
             p = 0.0;
             for (i = m-1; i >= l; i--)
                 {
                 f = s * e[i];
                 b = c * e[i];
                 if (fabs(f) >= fabs(g))
                    {
                    c = g / f;
                    r = sqrt((c * c) + 1.0);
                    e[i+1] = f * r;
                    c *= (s = 1.0/r);
                    }
                 else
                    {
                    s = f / g;
                    r = sqrt((s * s) + 1.0);
                    e[i+1] = g * r;
                    s *= (c = 1.0/r);
                    }
                 g = d[i+1] - p;
                 r = (d[i] - g) * s + 2.0 * c * b;
                 p = s * r;
                 d[i+1] = g + p;
                 g = c * r - b;
                 for (k = 1; k <= n; k++)
                     {
                     f = z[k][i+1];
                     z[k][i+1] = s * z[k][i] + c * f;
                     z[k][i] = c * z[k][i] - s * f;
                     }
                 }
                 d[l] = d[l] - p;
                 e[l] = g;
                 e[m] = 0.0;
             }
          }  while (m != l);
      }
 }

/*********************************/
/* Principal Components Analysis */
/*********************************/

/*********************************************************************/
/* Principal Components Analysis or the Karhunen-Loeve expansion is a
   classical method for dimensionality reduction or exploratory data
   analysis.  One reference among many is: F. Murtagh and A. Heck,
   Multivariate Data Analysis, Kluwer Academic, Dordrecht, 1987 
   (hardbound, paperback and accompanying diskette).

   This program is public-domain.  If of importance, please reference 
   the author.  Please also send comments of any kind to the author:

   F. Murtagh
   Schlossgartenweg 1          or        35 St. Helen's Road
   D-8045 Ismaning                       Booterstown, Co. Dublin
   W. Germany                            Ireland

   Phone:        + 49 89 32006298 (work)
                 + 49 89 965307 (home)
   Telex:        528 282 22 eo d
   Fax:          + 49 89 3202362
   Earn/Bitnet:  fionn@dgaeso51,  fim@dgaipp1s,  murtagh@stsci
   Span:         esomc1::fionn
   Internet:     murtagh@scivax.stsci.edu
   

   A Fortran version of this program is also available.     
    
   F. Murtagh, Munich, 6 June 1989                                   */   
/*********************************************************************/
//      printf("(filename -- give full path name,\n");
//      printf(" #rows                          \n");
//      printf(" #cols    -- integer values,\n");                  
//      printf(" option   -- R (recommended) for correlation analysis,\n");
//      printf("             V for variance/covariance analysis\n");
//      printf("             S for SSCP analysis.)\n");
//   n = atoi(argv[2]);              /* # rows  in pca.c*/  
//   m = atoi(argv[3]);              /* # columns  in pca.c */
//   strncpy(&option,argv[4],1);     /* Analysis option */


/* This is a different qsort algorithm

struct A { float v ; int i;};

int cmp(A *a1,A *a2) { return (a1->v < a2->v)?-1:((a1->v > a2->v)?1:0)  ; }

main() {
A *a;
a = malloc(sizeof(A) * nelements);
// copy data into +a;No match

for (j=0; j<nelements; j++) { a[j].v=value[j]; a[j].i=j; }

qsort(a,nelements,sizeof(a),cmp);

}
*/
