// ctfavg2.C Copyright 2003 Steve Ludtke
// This program will average all the particles in a file together
// including CTF correction. No alignment is performed.

//N ctfavg2
//F Averages a set of 2D images with ctf correction
//T LT
//1
//P <infile>	Set of images to be averaged
//P <outfile>	Output, 1 image will be appended
//P [res=<in A>]	Resolution in A if structure factor is not used
//P [ctfcw=<file>]	Structure factor file used to fit data in infile
//P [ctfc=<lp res>]	Lowpass filter resolution for ctf correction without wiener filter
//P [keepsig=<sig>]	Sigma multiplier for discarding particles
//P [keepval=<thr>]	Fixed threshold for discarding particles
//P [mask=<rad>]	Circular mask
//P [fileali]	Use alignments from file
//P [refali]	Do a single pass 2D alignment of each particle to the reference (ref0)
//P [copyeuler]	Used only if the filspec is cls####, will read the Eulers from proj.hed
//P [refmask]	Only for cls#### files, will mask based on proj.hed
//P [ref0]	First image is a reference image
//P [copyrefs]	Copy proj.hed image into the output file
//P [even|odd]	Uses only even or odd numbered particles
//P [snrmult=<val>]	Multiply the SNR by this number when Wiener filtering
//P [nolog]	Do not log a record of this program in .emanlog
//U ctfavg2 cls0000.lst classes.hed ctfcw=groel.xray
//D This program will average a set of prealigned images with CTF correction.

#include "config.h"
#include <stdlib.h>
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#include <string.h>
#include <math.h>
#include "List.h"

#include "EMData.h"

main(int argc, char** argv)
{
int i,j;
float filtr=25.0;
EMData av;
List *in;
XYData SF;
char *ctfcw=NULL;
int copyeuler=0,refmask=0,maskr=0,copyrefs=0,ref0=0,evenodd=0,nolog=0,fileali=0;
float snrmult=1.0,keepsig=0,keepval=0,ctfc=0.0;

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

for (i=3; i<argc; i++) {
	if (strncmp(argv[i],"res=",4)==0) filtr=atof(&argv[i][4]);
	else if (strncmp(argv[i],"copyeuler",9)==0) copyeuler=1;
	else if (strncmp(argv[i],"copyrefs",8)==0) copyrefs=1;
	else if (strncmp(argv[i],"refmask",7)==0) refmask=1;
	else if (strncmp(argv[i],"usefs",5)==0) ;
	else if (strncmp(argv[i],"quiet",5)==0) nolog=1;
	else if (strncmp(argv[i],"nolog",5)==0) nolog=1;
	else if (strncmp(argv[i],"even",4)==0) evenodd=1;
	else if (strncmp(argv[i],"odd",3)==0) evenodd=2;
	else if (strncmp(argv[i],"ref0",4)==0) ref0=1;
	else if (strncmp(argv[i],"snrmult=",8)==0) snrmult=atof(&argv[i][8]);
	else if (strncmp(argv[i],"keepsig=",8)==0) keepsig=atof(&argv[i][8]);
	else if (strncmp(argv[i],"keepval=",8)==0) keepval=PI*atof(&argv[i][8])/180.0;
	else if (strncmp(argv[i],"fileali",7)==0) fileali=1;
	else if (strncmp(argv[i],"refali",6)==0) fileali=-1;
	else if (strncmp(argv[i],"mask=",5)==0) maskr=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"ctfc=",5)==0) ctfc=atof(&argv[i][5]);
	else if (strncmp(argv[i],"ctfcw=",6)==0) {
		ctfcw=&argv[i][6];
	}
	else { printf("Unknown argument '%s'\n",argv[i]); exit(1); }
}

int Ref;
if (!nolog) {
	Ref=LOGbegin(argc,argv);
	LOG(Ref,argv[1],LOG_INFILE,NULL);
}

if (ctfcw) {
	SF.readFile(ctfcw);
	float *d=SF.getData();
	for (i=1; i<SF.n()*2; i+=2) d[i]=(d[i]<=0)?-4.0:log10(d[i]);
	SF.update();


//	for (i=0; i<SF.n();i++)
//		if (i==0) printf("x=%f y=%f \n", SF.x(i),SF.y(i));
}

if (evenodd) {
	// THIS CAUSED LATER ERRORS, I DO NOT KNOW WHY!
//	int ni=fileCount(argv[1]);
//	in=new List;
//	for (i=ref0+evenodd-1; i<ni; i+=2) {
//		EMData *im = new EMData();
//		if (im->readImage(argv[1],i)) error(ERR_CRIT,"Error reading image");
//		in->addObject(im);
//	}
	in=readImages(argv[1],ref0,-1);
	for (i=evenodd; i<in->count(); i++) delete in->removeObjectAt(i);
}
else in=readImages(argv[1],ref0,-1);

EMData *ref=new EMData();
if (ref0) {
	printf("Reading reference\n");
	ref->readImage(argv[1],0);
}

if (in->count()==0) {
	printf("No particles to average\n");
	exit(0);
}

if (!((EMData *)in->objectAt(0))->hasCTF()) {
	printf("These images don't appear to have CTF parameters\n");
	exit(1);
}

printf("Averaging %d images\n",in->count());
int nx=((EMData *)in->objectAt(0))->xSize();

if (fileali==1) {
	for (i=0; i<in->count(); i++) {
		EMData *t=(EMData *)in->objectAt(i);
		t->edgeNormalize();
//		printf("%f %f %f %d\n",t->alt(),t->Dx(),t->Dy(),t->isFlipped());
		t->rotateAndTranslate();
		if (t->isFlipped()) t->hFlip();
//		t->writeIMAGIC("zz.hed",-1);
	}
}
if (fileali==-1) {
	for (i=0; i<in->count(); i++) {
		EMData *t=(EMData *)in->objectAt(i);
		t->edgeNormalize();
//		printf("%f %f %f %d\n",t->alt(),t->Dx(),t->Dy(),t->isFlipped());
		EMData *e=t->RTFAlign(ref,NULL,1);
		e->refineAlign(ref);
		e->setParent(NULL);
		delete (EMData *)in->replaceObjectAt(i,e);
	}
}


av.setNImg(in->count());

EMData *mask=NULL;
EMData *masknorm=NULL;
int nf=0;
if (copyeuler || refmask || copyrefs) {
	ref->readImage(argv[1],0);
//	printf("Euler: %f  %f  %f\n",ref->alt()*180./PI,ref->az()*180./PI,ref->phi()*180./PI);
	ref->setNImg(-1);
	if (copyrefs) {
		sscanf(argv[1],"cls%d",&nf);
		ref->writeIMAGIC(argv[2],nf*2);
	}
	if (copyeuler) av.setRAlign(ref->alt(),ref->az(),ref->phi());
	if (refmask) {
		ref->realFilter(6);   // value != 0 -> 1
		masknorm=ref->copy(0,0);
		masknorm->addMaskShell(1);
		masknorm->subtract(ref);
		masknorm->addMaskShell(2);
		for (i=0; i<in->count(); i++) {
			EMData *t=(EMData *)in->objectAt(i);
			t->maskNormalize(masknorm,0);
			t->mult(ref);
		}
		mask=ref;
		ref=new EMData;
		ref->readImage(argv[1],0);
	}
}

if (maskr) {
	mask->setSize(nx,nx,1);
	mask->one();
	mask->applyMask(maskr,4);
	masknorm=mask->copy(0,0);
	masknorm->addMaskShell(1);
	masknorm->subtract(mask);
	masknorm->addMaskShell(2);
	for (i=0; i<in->count(); i++) {
		EMData *t=(EMData *)in->objectAt(i);
		t->maskNormalize(masknorm,0);
		t->mult(mask);
	}
}
float mean=0,sigma=0;

if ((keepsig||keepval)&&ref0) {
	for (i=0; i<in->count(); i++) {
		EMData *t=(EMData *)in->objectAt(i);
		float p=t->pcmp(ref,NULL);
		printf("\t%f",p*180./PI);
		if (keepval&&p>keepval) {
			in->removeObject(t);
			delete t;
			printf(" *");
		}
		if (i%5==4) printf("\n");
		mean+=p;
		sigma+=p*p;
	}
	mean/=(float)in->count();
	sigma=sqrt(sigma/(float)in->count()-mean*mean);
	printf("\nMean=%f   Sigma=%f\n",mean*180.0/PI,sigma*180.0/PI);
	if (keepsig) {
		printf("\nPass 2:\n");
		for (i=0; i<in->count(); i++) {
			EMData *t=(EMData *)in->objectAt(i);
			float p=t->pcmp(ref,NULL);
			printf("\t%f",p*180./PI);
			if (keepsig&&p>keepsig*sigma+mean) {
				in->removeObject(t);
				delete t;
				printf(" *");
			}
			if (i%5==4) printf("\n");
		}

	}
}
av.setNImg(in->count());


if (snrmult!=1.0) {
	for (i=0; i<in->count(); i++) {
		EMData *t=(EMData *)in->objectAt(i);
		if (!t) error(ERR_EXIT,"No object in SNRMULT");
		float *ctf=t->getCTF();
		if (!ctf) error(ERR_EXIT,"No CTF parms");
		ctf[2]*=snrmult;	// modify the array directly, a bit risky if the library is revamped
	}
}

if (ctfcw) av.makeAverageCTFCW(in,&SF);
else if (ctfc>0) av.makeAverageCTFC(in,ctfc);
else av.makeAverage(in);
if (refmask ||mask) {
	av.maskNormalize(masknorm,0);
	av.mult(mask);
}

if (copyeuler&&copyrefs) av.writeIMAGIC(argv[2],nf*2+1);
else if (copyeuler) av.writeIMAGIC(argv[2],nf);
else av.writeIMAGIC(argv[2],-1);

if (!nolog) {
	LOG(Ref,argv[2],LOG_OUTFILE,NULL);
	LOG(Ref,NULL,LOG_END,NULL);
}
}

