// imgdump.C 1/03 Liwei Peng

//N imgdump
//F Dump an image's dimensions and raw data into a file.
//P <imgfile> image filename

#include <stdio.h>
#include <string.h>
#include "EMData.h"

int main(int argc, char* argv[])
{
    if (argc != 2) {
	fprintf(stderr, "\nusage: %s imagefile\n\n", argv[0]);
	return 1;
    }

    const char* imagefile = argv[1];
    int imgfile_sz = strlen(imagefile);
    
    char* datafilename = new char[imgfile_sz + 10];
    sprintf(datafilename, "%s.dat", imagefile);

    FILE* datafile = fopen(datafilename, "wb");
    if (!datafile) {
	fprintf(stderr, "Error: cannot open data file '%s'\n",
		datafilename);
	return 1;
    }
    
    EMData* e = new EMData();
    e->readImage(imagefile, 0);
    int nx = e->xSize();
    int ny = e->ySize();
    int nz = e->zSize();
    
    fprintf(datafile, "image file: %s\n", imagefile);
    fprintf(datafile, "x size = %d\n", nx);
    fprintf(datafile, "y size = %d\n", ny);
    fprintf(datafile, "z size = %d\n", nz);
    
    
    float* data = e->getDataRO();
    int size = nx * ny * nz;
    int x_sz = nx * sizeof(float);
    
    for (int i = 0; i < size; i += nx) {
	fwrite(&data[i], x_sz, 1, datafile);
    }
    
    e->doneData();
    
    fclose(datafile);

    delete e;
    e = 0;

    delete [] datafilename;
    datafilename = 0;

    return 0;
}
